import { generateHlsSession, getVideoInfo } from './api';
import { UserAgent } from './constants';
import { cookieStringToObj, downloadAesM3u8, getMediaPair, objToCookieString } from './utils';
import fs from 'fs';

const videoId = 'sm11448603';
(async () => {
	const videoInfo = await getVideoInfo(videoId);
	// console.log(videoInfo);
	const videoSession = await generateHlsSession(videoId, videoInfo);
	// console.log(videoSession);

	const expireTime = new Date(videoSession.data.data.expireTime);
	const currentTime = new Date();
	if (expireTime < currentTime) {
		console.error('Session expired');
		return;
	}

	const highestVideo = videoInfo.data.media.domand.videos.sort((a, b) => b.qualityLevel - a.qualityLevel)[0];
	const highestAudio = videoInfo.data.media.domand.audios.sort((a, b) => b.qualityLevel - a.qualityLevel)[0];

	console.log(videoSession.data.data.contentUrl);
	const cookie = objToCookieString({
		nicosid: videoInfo.data.client.nicosid,
		domand_bid: cookieStringToObj(videoSession.cookie).domand_bid,
	});
	const m3u8 = await fetch(videoSession.data.data.contentUrl, {
		headers: {
			Cookie: cookie,
			'User-Agent': UserAgent,
		},
	}).then(res => res.text());

	// console.log(m3u8);

	const mediaPair = getMediaPair(highestVideo.id, highestAudio.id, m3u8);
	// console.log(JSON.stringify(mediaPair, null, 2));

	const video = await downloadAesM3u8(mediaPair.video?.uri, cookie);
	fs.writeFileSync('video.mp4', Buffer.from(video));

	// console.log(mediaPair.audio);
	const audio = await downloadAesM3u8(mediaPair.audio['Main Audio'].uri, cookie);
	fs.writeFileSync('audio.mp4', Buffer.from(audio));

})();
